﻿using System;
using System.Windows.Forms;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;


namespace Test
{
    public partial class TestForm : Form
    {
        private CustomRotationProjection projection;
        private int i = 0;
        Timer timer = new Timer();

        public TestForm()
        {
            InitializeComponent();
            timer.Interval = 2000;
            timer.Tick += new EventHandler(timer_Tick);
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            InMemoryFeatureLayer lineLayer = (InMemoryFeatureLayer)winformsMap1.FindFeatureLayer("LineLayer");
            LineShape line = (LineShape)lineLayer.InternalFeatures[0].GetShape();
            if (i < line.Vertices.Count)
            {
                // determine your own pivot and angle
                projection.PivotVertex = line.Vertices[i - 1 < 0 ? 0 : i - 1];
                double angle = 20;
                projection.Angle += angle;
            }
            else
            {
                i = 0;
                projection.PivotVertex = line.Vertices[0];
                projection.Angle = 0;
            }
            PointShape point = (PointShape)projection.ConvertToExternalProjection(new PointShape(line.Vertices[i].X, line.Vertices[i].Y));
            winformsMap1.CenterAt(point);
            winformsMap1.CurrentExtent = projection.GetUpdatedExtent(winformsMap1.CurrentExtent);
            winformsMap1.Refresh(winformsMap1.Overlays["WorldOverlay"]);
            i++;
        }

        private void TestForm_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.Meter;
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);

            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(@"..\..\Data\Countries02.shp");
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            projection = new CustomRotationProjection(ManagedProj4Projection.GetEpsgParameters(4326), ManagedProj4Projection.GetGoogleMapParameters());
            worldLayer.Open();
            projection.PivotVertex = new Vertex(worldLayer.GetBoundingBox().GetCenterPoint());
            worldLayer.Close();
            worldLayer.FeatureSource.Projection = projection;

            InMemoryFeatureLayer lineLayer = new InMemoryFeatureLayer();
            lineLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.InternationalBorder1;
            lineLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            LineShape line = new LineShape();
            line.Vertices.Add(new Vertex(10, 10));
            line.Vertices.Add(new Vertex(15, 20));
            line.Vertices.Add(new Vertex(25, 35));
            line.Vertices.Add(new Vertex(40, 40));
            line.Vertices.Add(new Vertex(30, 30));
            line.Vertices.Add(new Vertex(40, 20));
            line.Vertices.Add(new Vertex(50, 0));
            line.Vertices.Add(new Vertex(70, -10));
            lineLayer.InternalFeatures.Add(line.GetFeature());
            lineLayer.FeatureSource.Projection = projection;

            InMemoryFeatureLayer pointLayer = new InMemoryFeatureLayer();
            pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.Capital1;
            pointLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            foreach (Vertex vertex in line.Vertices)
            {
                pointLayer.InternalFeatures.Add((new PointShape(vertex.X, vertex.Y)).GetFeature());
            }
            pointLayer.FeatureSource.Projection = projection;

            LayerOverlay worldOverlay = new LayerOverlay();
            worldOverlay.Layers.Add("WorldLayer", worldLayer);
            worldOverlay.Layers.Add("LineLayer", lineLayer);
            worldOverlay.Layers.Add("PointLayer", pointLayer);
            winformsMap1.Overlays.Add("WorldOverlay", worldOverlay);

            winformsMap1.ZoomLevelSnapping = ZoomLevelSnappingMode.None;
            lineLayer.Open();
            winformsMap1.CurrentExtent = lineLayer.GetBoundingBox();
            lineLayer.Close();

            winformsMap1.Refresh();
            timer.Start();
        }
    }
}
